from pip2spack.actions.action import Action
from pip2spack.framework.helpers import create_directory, create_package
from pip2spack.framework.messages import Messages
from pip2spack.core.pypi_package import PyPiPackage
from pip2spack.core.verification import Verification


class CreateAction(Action):
    ACTION = "create"
    PARAM_NAME = "ACTION"

    def fill_parser_arguments(self):
        self.parser.add_argument('name', type=str, nargs='+', help='Package name on the pypi.org')

    def process_action(self, configuration):
        ready_packages = Verification(configuration.name).available_packages
        self.generate_packages(ready_packages)

    @staticmethod
    def generate_packages(ready_packages):
        for key, value in ready_packages.items():
            Messages.info(f"Generating package for the {key}")

            s_package = PyPiPackage(value)
            generated_file = s_package.generate_file()

            create_directory(key)
            created_package_uri = create_package(key, generated_file)
            print(f" OK  :: Package for {key} was generated\n\t "
                  f":: If \'spack install py-{key}\' return any problems, try to uncomment:\n\t "
                  f":: \tdepends_on('py-setuptools', type='build')\n\t "
                  f":: at {created_package_uri}")






from pip2spack.actions.action import Action
from pip2spack.framework.messages import Messages
from pip2spack.core.pypi_package import PyPiPackage
from pip2spack.core.verification import Verification


class DownloadAction(Action):
    ACTION = "download"
    PARAM_NAME = "ACTION"

    def fill_parser_arguments(self):
        self.parser.add_argument('name', type=str, nargs='+', help='Package name on the pypi.org')

    def process_action(self, configuration):
        ready_packages = Verification(configuration.name).available_packages
        self.download(ready_packages)

    @staticmethod
    def download(ready_packages):
        for key, value in ready_packages.items():
            Messages.info(f"Download latest package for the {key}")

            d_package = PyPiPackage(value)
            d_package.download_versions()



import os
from dataclasses import dataclass
from typing import AnyStr

from pip2spack.core.spack_repository import SpackRepository
from pip2spack.framework import constants as constants
from pip2spack.framework.messages import Messages
from pip2spack.core.pypi_package import PyPiPackage


@dataclass
class Package:
    package_name: AnyStr
    package_path: AnyStr = None
    _raw_package: AnyStr = None

    versions = []

    def __post_init__(self):
        s = SpackRepository()
        self.package_path = s.get_package_path(package_name=self.package_name)
        self._open()

    def _open(self):
        with open(self.package_path, 'r') as package_py:
            self._raw_package = package_py.read()

    def replace_structure_with_marker(self):
        self.modificated_package = self._raw_package.split('\n')
        if not isinstance(self.modificated_package, list):
            raise

        rows_with_version = []
        for index, content in enumerate(self.modificated_package):
            if "version(" in content:
                rows_with_version.append((index, content.lstrip().rstrip()))

        rows_with_version.reverse()  # start removing a empty rows from bottom file
        for index, row in enumerate(rows_with_version):
            if index == 0:
                self.modificated_package[row[0]] = self.modificated_package[row[0]].replace(row[1],
                                                                                            constants.MARKER_VERSION)
            else:
                del self.modificated_package[row[0]]

        self.replace_marker_with_template()

    def replace_marker_with_template(self):
        rows_with_version = []
        for index, content in enumerate(self.modificated_package):
            if constants.MARKER_VERSION in content:
                rows_with_version.append(index)

        if not [x for x in self.modificated_package if constants.MARKER_VERSION in x]:
            Messages.error(f"{self.package_name} :: pip2spack can not find a \'version\' tag.\n\t Please create a new package with:\n\t  pip2spack create {self.package_name}")
            return

        rows_with_version.reverse()  # start removing a empty rows from bottom file
        for index, row in enumerate(rows_with_version):
            if index == 0:
                spaces = self._count_trailing_spaces(self.modificated_package[row])
                self.modificated_package[row] = \
                    self.modificated_package[row].replace(constants.MARKER_VERSION,
                                                          constants.marker_version_template(spaces))
            else:
                break

        self.generate_modded_package(save=True)
        if not self.save(self.update_newest_versions(self.package_name)):
            Messages.error(f"Something went wrong, package {self.package_name} was not updated")
        else:
            Messages.ok(f"{self.package_name} :: builtin package was updated at {self.package_path}")

    def update_newest_versions(self, package_name):
        sp = PyPiPackage(package_name)
        file = sp.generate_custom_file(os.path.dirname(os.path.realpath(self.package_path)), sp._get_versions)
        return file

    def save(self, new_file):
        with open(self.package_path, 'w') as package:
            package.write(new_file)
            return True

    def generate_modded_package(self, save: bool = False):
        if not save:
            return '\n'.join(self.modificated_package).lstrip()
        self.save('\n'.join(self.modificated_package).lstrip())

    @staticmethod
    def available_markers(marker_name):
        return {"version": constants.MARKER_VERSION}.get(marker_name, None)

    @staticmethod
    def _count_trailing_spaces(sentence) -> int:
        return len(sentence) - len(sentence.lstrip())



class Messages:
    @staticmethod
    def package_availability(available: list, unavailable: list):
        Messages.info("Package status")
        if available:
            print("Available:")
            for key in available:
                print(f"\t{key}")

        packages_not_found = unavailable
        if not packages_not_found:
            print()
            return

        print("\nUnavailable:")
        for p in packages_not_found:
            print(f'\t{p}')
        else:
            print()

    @staticmethod
    def error(message):
        print(f"ERRO :: {message}")

    @staticmethod
    def warn(message):
        print(f"WARN :: {message}")

    @staticmethod
    def info(message):
        print(f"INFO :: {message}")

    @staticmethod
    def ok(message):
        print(f" OK  :: {message}")

    @staticmethod
    def clean(message):
        print(f"     :: {message}")



from pip2spack.actions.action import Action
from pip2spack.core.package import Package
from pip2spack.core.verification import Verification


class UpdateAction(Action):
    ACTION = "update"
    PARAM_NAME = "ACTION"

    def fill_parser_arguments(self):
        self.parser.add_argument('name', type=str, nargs='+', help='Package name on the pypi.org')

    def process_action(self, configuration):
        ready_packages = Verification(configuration.name).available_packages
        self.update_packages(ready_packages)

    @staticmethod
    def update_packages(package_names):
        if isinstance(package_names, dict):
            package_names = [k for k, v in package_names.items()]

        for p_name in package_names:
            package = Package(package_name=p_name)
            package.replace_structure_with_marker()
            package.generate_modded_package()



import json

import requests

from pip2spack.framework.messages import Messages


class Verification:
    def __init__(self, arguments: list):
        self.__ready_packages = self.validate_pip_package_exists(arguments)
        self.show_packages_for_process(self.__ready_packages, arguments)

    @property
    def available_packages(self):
        return self.__ready_packages

    @staticmethod
    def validate_pip_package_exists(potential_packages):
        if not potential_packages:
            return

        packages_status = {}
        for package in potential_packages:
            status = requests.get(f'https://pypi.org/pypi/{package}/json')
            if status.status_code != 200:
                continue
            packages_status[package] = json.loads(status.content)
        return packages_status

    @staticmethod
    def show_packages_for_process(package_ready, all_packages):
        """Display at the terminal a information about arability of provided package names"""
        packages_not_found = [x for x in all_packages if x not in package_ready]
        packages_found = [k for k, v in package_ready.items()]
        Messages.package_availability(packages_found, packages_not_found)
